require_relative 'project'
require_relative 'die'
require_relative 'funding_round'
require_relative 'pledge_pool'

require 'csv'

module CrowdFund
  class FundRequest
    def initialize(title)
      @title = title
      @projects = []
    end

    def add_project(project_name)
      @projects << project_name
    end

    def load_projects(file)
      CSV.foreach(file) do |row|
        add_project(Project.new(row[0], row[2].to_i, row[1].to_i))
      end
    end

    def save_to_hit_target_stats(output_file)
      under_fund_projects = @projects.reject(&:full?)
      File.open(output_file, "w") do |file|
        under_fund_projects.sort.each do |project|
          file.puts to_hit_target_entry(project)
        end
      end
    end

    def to_hit_target_entry(project)
      "#{project.name.ljust(50, '.')} (#{project.total_to_fund} to hit target)"
    end

    def fully_fund_stats(fully_fund_list)
      puts "\n#{fully_fund_list.size} fully fund projects:"
      fully_fund_list.each do |project|
        puts "#{project.name.ljust(10, '.')} (#{project.amount})"
      end
    end

    def total_pledges_given
      @projects.reduce(0) { |pledges, project| pledges + project.total_pledges }
    end

    def under_fund_stats(under_fund_list)
      puts "\n#{under_fund_list.size} under fund projects:"
      under_fund_list.each do |project|
        puts "#{project.name.ljust(10, '.')} (#{project.amount})"
      end
    end

    def print_stats
      puts "\nAfter all rounds of funds:"
      puts "\n#{total_pledges_given} pledges given."
      @projects.each do |project|
        puts "\nProject #{project.name} total pledges:"
        project.each_pledge_received do |pledge|
          puts "$#{pledge.value} on #{pledge.name} pledges."
        end
      end
      fully_fund, under_fund = @projects.partition(&:full?)
      fully_fund_stats(fully_fund)
      under_fund_stats(under_fund)
    end

    def print_to_hit_target_stats
      sorted_projects = @projects.reject(&:full?)
      puts "\nList of projects that still need contributions:"
      sorted_projects.sort.each do |project|
        puts to_hit_target_entry(project)
      end
    end

    def request_funding(rounds)
      PledgePool.list
      @projects.each do |project|
        puts project
      end
      1.upto(rounds) do |round|
        puts "\nFunding round number #{round}:"
        @projects.each do |project|
          FundingRound.round_of_funds(project)
          puts project
        end
      end
    end
  end

  if __FILE__ == $0
    funding_scenario = Fund_Request.new("first_funding")
    funding_scenario.add_project(Project.new("ProJect", 5000, 10000))
    funding_scenario.request_funding
  end
end
