require_relative 'pledge_pool'
require_relative 'fundable'
require_relative 'printable'

module CrowdFund
  class Project

    include Fundable
    include Printable

    attr_accessor :name, :amount, :target

    def initialize(name, amount=0, target=1000)
      @name = name.upcase
      @amount = amount
      @target = target
      @pledges_received = Hash.new(0)
    end

    def total_pledges
      @pledges_received.values.reduce(0, :+)
    end

    def each_pledge_received
      @pledges_received.each do |name, value|
        yield Pledge.new(name, value)
      end
    end

    def receive_pledge(pledge)
      @pledges_received[pledge.name] += pledge.value
      @amount += total_pledges
      puts "#{@name} received a #{pledge.name} pledge worth $#{pledge.value}."
      puts "Project #{@name} pledges: #{@pledges_received}"
    end
  end

  if __FILE__ == $PROGRAM_NAME
    project1 = Project.new("new_project", 5000, 150000)
    puts project1
    3.times do
      project1.add(4000)
    end
    project1.remove(10000)
    puts project1
  end
end
